
#include <Adafruit_NeoPixel.h>
#include <Wire.h> // must be included here so that Arduino library object file references work
#include <RtcDS3231.h>
#include <EasyButton.h>

#include "CircularBufferLib.h"  // om snake te kunnen spelen
RtcDS3231<TwoWire> Rtc(Wire);

#define LED_PIN     6
#define LED_COUNT  110
int BrightnessPreset = 170;
const byte NummersLocatie[] = {99, 91, 89, 82, 47, 94, 55, 66, 40, 61, 51, 85, 71}; //Positie Eerste letter van alle nummers 0,1,2,3
const byte NummersLengte[] =  { 3,  3,  4,  4,  4,  4,  3,  5,  4,  5,  4,  3,  6};  //lengte nummers 0,1,2,3
byte RandomLetters[10];
byte NuMinuten = 0;
byte NuUren = 0;
byte Klokmodus = 0; // 0= klok; 1=Emma Roger; 2 = vallende letters
int NuUurHash = 1;
int CycleTime = 400; //Update time voor de klok
int CycleTimeDefault = 400; // standaard Update time voor de klok
unsigned long time_NextCycle = 0;
unsigned long time_NextCycleRTC = 0;
unsigned long time_now = 0;

const byte InputButton1 = 5;
const byte InputButton2 = 3;
const byte InputButton3 = 4;
const byte InputButton4 = 2;
//byte Button1Status, Button2Status, Button3Status, Button4Status;

const String VerjaardagNamen[] = {"", "J", "K", "D", "W"};
const int VerjaardagMaanden[] = {"", 4, 5, 1, 5};
const int VerjaardagDagen[] = {"", 26, 24 , 23, 20};
byte VerjaardagVandaag = 0;
byte VerjaardagVervers = 20;  // verwisseltijd in seconden, maximum is dertig
int NuJaar;
byte NuMaand;
byte NuDag;
byte NuUur;
byte NuMinuut;
byte NuSeconden;
byte HuwelijkMaand = 3;
byte HuwelijkDag = 5;
int HuwelijkJaar = 2020;
byte HuwelijkUur = 12;

byte SetItem = 0; // 0=Uur, 1=Minuut, 2=Dag, 3=Maand, 4=jaar
int SetUur;
int SetMinuut;
int SetDag;
int SetMaand;
int SetJaar;

bool ontploft = false;

RtcDateTime TijdNu;


EasyButton button1(InputButton1);
EasyButton button2(InputButton2);
EasyButton button3(InputButton3);
EasyButton button4(InputButton4);


// Declare our NeoPixel strip object:
Adafruit_NeoPixel strip(LED_COUNT, LED_PIN, NEO_GRBW + NEO_KHZ800);

void setup() {
  /*
    //knoppen initialiseren
    pinMode(Button1, INPUT_PULLUP); // Mode button
    pinMode(Button2, INPUT_PULLUP); // Set button
    pinMode(Button3, INPUT_PULLUP); // Up button
    pinMode(Button4, INPUT_PULLUP); // Down button
  */
  // Initialize the 4 buttons
  button1.begin();
  button3.begin();
  button4.begin();
  // Add the callback function to be called when the button1 is pressed.
  button1.onPressed(onButton1Pressed);
  button3.onPressed(onButton3Pressed);
  button4.onPressed(onButton4Pressed);
  button1.onPressedFor(2000, onButton1LongPress);


  strip.begin();           // INITIALIZE NeoPixel strip object (REQUIRED)
  strip.show();            // Turn OFF all pixels ASAP
  strip.setBrightness(BrightnessPreset); // Set BRIGHTNESS to about 1/5 (max = 255)
  // Nu alles voor RTC: gewoon copy-paste van DS3231_Simple van Makuna
  Serial.begin(9600);

  Serial.print("compiled: ");
  Serial.print(__DATE__);
  Serial.println(__TIME__);

  //--------RTC SETUP ------------
  // if you are using ESP-01 then uncomment the line below to reset the pins to
  // the available pins for SDA, SCL
  // Wire.begin(0, 2); // due to limited pins, use pin 0 and 2 for SDA, SCL

  Rtc.Begin();

  RtcDateTime compiled = RtcDateTime(__DATE__, __TIME__);
  //    printDateTime(compiled);
  Serial.println();

  if (!Rtc.IsDateTimeValid())
  {
    if (Rtc.LastError() != 0)
    {
      // we have a communications error
      // see https://www.arduino.cc/en/Reference/WireEndTransmission for
      // what the number means
      Serial.print("RTC communications error = ");
      Serial.println(Rtc.LastError());
    }
    else
    {
      // Common Causes:
      //    1) first time you ran and the device wasn't running yet
      //    2) the battery on the device is low or even missing

      Serial.println("RTC lost confidence in the DateTime!");

      // following line sets the RTC to the date & time this sketch was compiled
      // it will also reset the valid flag internally unless the Rtc device is
      // having an issue

      Rtc.SetDateTime(compiled);
    }
  }

  if (!Rtc.GetIsRunning())
  {
    Serial.println("RTC was not actively running, starting now");
    Rtc.SetIsRunning(true);
  }

  RtcDateTime now = Rtc.GetDateTime();

  //Timestamp naar serial monitor sturen
  Serial.print("RTC Timestamp: ");
  Serial.print(now.Year());
  Serial.print("-");
  Serial.print(now.Month());
  Serial.print("-");
  Serial.print(now.Day());
  Serial.print(" time: ");
  Serial.print(now.Hour());
  Serial.print(":");
  Serial.println(now.Minute());

  if (now < compiled)
  {
    Serial.println("RTC is older than compile time!  (Updating DateTime)");
    Rtc.SetDateTime(compiled);
  }
  else if (now > compiled)
  {
    Serial.println("RTC is newer than compile time. (this is expected)");
  }
  else if (now == compiled)
  {
    Serial.println("RTC is the same as compile time! (not expected but all is fine)");
  }

  // never assume the Rtc was last configured by you, so
  // just clear them to your needed state
  Rtc.Enable32kHzPin(false);
  Rtc.SetSquareWavePin(DS3231SquareWavePin_ModeNone);

  time_NextCycle = millis() + CycleTime;
  time_NextCycleRTC = millis() + (CycleTime * 100) ; //RTC maar af en toe checken
  /*Serial.print(time_NextCycle );
    Serial.print(" -");
    Serial.print(CycleTime );
    Serial.print(" -");
    Serial.println(time_now );
  */
  TijdOpLedAanpassen(true);
}

void loop() {
  button1.read(); // Continuously read the status of the buttons
  button3.read();
  button4.read();
  time_now = millis(); //Millis gebruiken voor timing
  if ((time_NextCycle - time_now) >= 10000) { //als millis reset next cycle ook resetten
    time_NextCycle = time_now + CycleTime;
    time_NextCycleRTC = time_now + (CycleTime * 100) ; //RTC maar af en toe checken

  }
  if (time_now >= time_NextCycleRTC) {
    RTCTest();
    //Opzoeken of vandaag iemand jarig is
    VerjaardagVandaag = 0;
    TijdNu = Rtc.GetDateTime();
    NuMaand = TijdNu.Month();
    NuDag = TijdNu.Day();
    NuJaar = TijdNu.Year();
    NuUur = TijdNu.Hour();
    NuMinuut = TijdNu.Minute();
    for (int PersoonX = 1; PersoonX < 5; PersoonX++) { //zien of er iemand jarig is
      if ((NuMaand == VerjaardagMaanden[PersoonX]) && (NuDag == VerjaardagDagen[PersoonX])) {
        VerjaardagVandaag = PersoonX;
      }
    }
    //Nakijken of het een huwelijksverjaardag is
    /*
      if ((NuMaand == HuwelijkMaand) && (NuDag == HuwelijkDag) && (NuJaar > HuwelijkJaar)) { // exacte huwelijksverjaardag
      if ((NuJaar - HuwelijkJaar) > 12) {
        VerjaardagVandaag = 10;
      }
      else {
        VerjaardagVandaag = 99; //99 is standaard conditie
      }
      }
    */
    /*
        if ((NuMaand == HuwelijkMaand) && (NuDag == HuwelijkDag) && (NuJaar = HuwelijkJaar)) {  // huwelijksdag zelf
          if (NuUur > HuwelijkUur) {
            VerjaardagVandaag = 11;
          }
          else {
            VerjaardagVandaag = 99; //99 is standaard conditie
          }
        }

        if ((NuMaand == HuwelijkMaand) && (NuDag > HuwelijkDag) && ((NuDag - HuwelijkDag) < 13) && (NuJaar = HuwelijkJaar)) { // Eerste dagen
          VerjaardagVandaag = 12;
        }
        if ((NuJaar == HuwelijkJaar) && (NuDag == HuwelijkDag) && ((NuMaand - HuwelijkMaand) > 0) ) { // Eerste maanden
          VerjaardagVandaag = 13;
        }
    */



    time_NextCycleRTC = time_now + (CycleTime * 100) ; //RTC maar af en toe checken

  }

  if (time_now >= time_NextCycle) {
    time_NextCycle = time_now + CycleTime;
    switch (Klokmodus) { //Naargelang de modus iets anders doen
      case 0: TijdOpLedAanpassen(true);     break;
      case 1: JKDW();     break;
      case 2:  if (!ontploft) {
          VanuitRingenOntploffen();
        } ontploft = true; break;
        //default: UurNaarLED(14, 25, true , true); NuUurHash = 3; break;
    }
  }
}

void TijdDatumInstellen(byte Knop) { // 0=niets, 1=SetKnop Lang, 2 = Setknop Kort, 3=Knop boven, 4=knop onder
  if ((SetItem == 1 || SetItem == 2) && (Knop == 1)) { // tijdens tijdsinstellen skippen naar datuminstellen JAAR bij lang duwen
    SetItem = 3; //  0=niets, 1=Uur, 2=Minuut, 3=jaar, 4=Maand, 5=Dag
    Klokmodus = 4; //Modus instellen op TijdDatumInstellen
    strip.clear();         //   Set all pixels in RAM to 0 (off)
       byte Jaarletters[] = {77, 88, 80, 81};
    RandomLettersNaarLed(Jaarletters, 4, true, 0);
    delay (2000); // even wachten zodat je kan zien wat je instelt
    TijdNu = Rtc.GetDateTime();
    SetJaar = TijdNu.Year();
    strip.clear();         //   Set all pixels in RAM to 0 (off)
    JaarNaarLED (SetJaar, true, true);
  }

  if ((SetItem == 0) && (Knop == 1)) { // Als er nog geen setitem is beginnen met de tijd/uur in te stellen
    SetItem = 1; // 0=niets, 1=Uur, 2=Minuut, 3=jaar, 4=Maand, 5=Dag
    Klokmodus = 4; //Modus instellen op TijdDatumInstellen
    strip.clear();         //   Set all pixels in RAM to 0 (off)
    TekstNaarLed (103, 3, true, 0); //uur
    delay (2000); // even wachten zodat je kan zien wat je instelt
    TijdNu = Rtc.GetDateTime();
    SetMinuut = TijdNu.Minute();
    SetUur = TijdNu.Hour();
    strip.clear();         //   Set all pixels in RAM to 0 (off)
    UurNaarLED(SetUur, SetMinuut, false, false); // Startuur tonen
  }
  if ((Knop == 2) && (SetItem == 5)) { // Datum instellen beëindigen
    SetItem = 0; //  0=niets, 1=Uur, 2=Minuut, 3=jaar, 4=Maand, 5=Dag
    strip.clear();         //   Set all pixels in RAM to 0 (off)
    Klokmodus = 0; //Modus instellen op TijdDatumInstellen
    NuUurHash = -1;
    TijdNu = Rtc.GetDateTime();
    Rtc.SetDateTime(RtcDateTime(SetJaar, SetMaand, SetDag, TijdNu.Hour(), TijdNu.Minute(), 00));//Tijd in RTC aanpassen
  }

  if ((Knop == 2) && (SetItem == 2)) { // Tijdinstellen beëindigen
    SetItem = 0; //  0=niets, 1=Uur, 2=Minuut, 3=jaar, 4=Maand, 5=Dag
    strip.clear();         //   Set all pixels in RAM to 0 (off)
    Klokmodus = 0; //Modus instellen op TijdDatumInstellen
    NuUurHash = -1;
    TijdNu = Rtc.GetDateTime();
    Rtc.SetDateTime(RtcDateTime(TijdNu.Year(), TijdNu.Month(), TijdNu.Day(), SetUur, SetMinuut, 00));//Tijd in RTC aanpassen
  }
  if ((Knop == 2) && (SetItem == 1)) { // Skippen naar Minuten Instellen
    SetItem = 2; //  0=niets, 1=Uur, 2=Minuut, 3=jaar, 4=Maand, 5=Dag
    strip.clear();         //   Set all pixels in RAM to 0 (off)
    byte Minuutletters[] = {3, 17, 18};
    RandomLettersNaarLed(Minuutletters, 3, true, 0);
    delay (2000); // even wachten zodat je kan zien wat je instelt
    strip.clear();         //   Set all pixels in RAM to 0 (off)
    UurNaarLED(SetUur, SetMinuut, false, false); // Startuur tonen
  }

  if ((Knop == 2) && (SetItem == 4)) { // Skippen naar Dag Instellen
    SetItem = 5; //  0=niets, 1=Uur, 2=Minuut, 3=jaar, 4=Maand, 5=Dag
    strip.clear();         //   Set all pixels in RAM to 0 (off)
    //TekstNaarLed (96, 3, true, 0); //dag
    byte Dagletters[] = {58, 59, 63};
    RandomLettersNaarLed(Dagletters, 3, true, 0);
    delay (2000); // even wachten zodat je kan zien wat je instelt
    TijdNu = Rtc.GetDateTime();
    SetDag = TijdNu.Day();
    strip.clear();         //   Set all pixels in RAM to 0 (off)
    NummerNaarLed(SetDag, false, 0);
  }


  if ((Knop == 2) && (SetItem == 3)) { // Skippen naar Maand Instellen
    SetItem = 4; //  0=niets, 1=Uur, 2=Minuut, 3=jaar, 4=Maand, 5=Dag
    strip.clear();         //   Set all pixels in RAM to 0 (off)
    byte Maandletters[] = {39, 59, 60, 61, 82};
    RandomLettersNaarLed(Maandletters, 5, true, 0);
    delay (2000); // even wachten zodat je kan zien wat je instelt
    TijdNu = Rtc.GetDateTime();
    SetMaand = TijdNu.Month();
    strip.clear();         //   Set all pixels in RAM to 0 (off)
    NummerNaarLed(SetMaand, false, 0);
  }

  if (Knop == 3) { //Knop naar boven geduwd
    if (SetItem == 1) { //uur instellen
      SetUur++;
    }
    else if (SetItem == 2) { //minuten instellen
      SetMinuut = SetMinuut + 5;
      if (SetMinuut > 59) {
        SetMinuut = 0;
        SetUur++;
      }
    }
    else if (SetItem == 3) { //Jaren instellen
      SetJaar++;
    }
    else if (SetItem == 4) { //Maanden instellen
      SetMaand++;
    }
    else if (SetItem == 5) { //Maanden instellen
      SetDag++;
    }
    byte DagMax = 31;
    if (SetMaand == 4  || SetMaand == 6  || SetMaand == 9  || SetMaand == 11) {
      DagMax = 30;
    } else if (SetMaand == 2) {
      DagMax = 29;
    }
    if (SetDag > DagMax ) {
      SetDag = DagMax;
    }
    if (SetUur > 23) {
      SetUur = 0;
    }
    if (SetMaand > 12) {
      SetMaand = 12;
    }
    strip.clear();         //   Set all pixels in RAM to 0 (off)
    //UurNaarLED(SetUur, SetMinuut, false, false); // Startuur tonen
  }

  if (Knop == 4) { //Knop naar onder geduwd
    if (SetItem == 1) { //uur instellen
      SetUur--;
    }
    else if (SetItem == 2) { //minuten instellen
      SetMinuut = SetMinuut - 5;
      if (SetMinuut < 0) {
        SetMinuut = 55;
        SetUur--;
      }
    }
    else if (SetItem == 3) { //jaren instellen
      SetJaar--;
      if (SetJaar < 2020) {
        SetJaar = 2020;
      }
    }
    else if (SetItem == 4) { //maanden instellen
      SetMaand--;
      if (SetMaand < 1) {
        SetMaand = 1;
      }
    }
    else if (SetItem == 5) { //dagen instellen
      SetDag--;
      if (SetDag < 1) {
        SetDag = 1;
      }
    }
    if (SetUur < 0) {
      SetUur = 23;
    }
    strip.clear(); //   Set all pixels in RAM to 0 (off)
  }

  if (Knop == 3 || Knop == 4 ) { //Knop naar onder of boven geduwd

    switch (SetItem) {
      case 1: UurNaarLED(SetUur, SetMinuut, false, false);    break;  // SetUur tonen
      case 2: UurNaarLED(SetUur, SetMinuut, false, false);    break;  // SetUur tonen
      case 3: JaarNaarLED (SetJaar, false, false);       break;  // SetUur tonen
      case 4: NummerNaarLed(SetMaand, false, 0);       break; // SetUur tonen
      case 5: NummerNaarLed(SetDag, false, 0);       break; // SetUur tonen
    }
  }

}

void JaarNaarLED (int jaar, bool Anim20, bool Anim) {
  int Teller = jaar - 2020;
  TekstNaarLed (15, 7, Anim20, 0);  //20
  if (Teller == 0) {
    ;// niets doen voor jaar 2020
  }
  else if (Teller > 0 && Teller < 11) {
    NummerNaarLed(Teller, Anim, 0);
  }
  else if (Teller > 10 && Teller < 23) {
    TekstNaarLed (22, 4, Anim20, 0);  //10
    NummerNaarLed(Teller - 10, Anim, 0);
  }
  else if (Teller > 22 && Teller < 30) {
    TekstNaarLed (22, 4, Anim20, 0);  //10
    TekstNaarLed (83, 4, Anim20, 0);  //10bis
    NummerNaarLed(Teller - 20, Anim, 0);
  }
  Serial.print("Teller: ");
  Serial.print(Teller);
  Serial.print(" SetJaar: ");
  Serial.println(SetJaar);

}

// Callback function to be called when button1 is pressed
void onButton1Pressed() {
  Serial.println("Button1 has been pressed!");
  if (Klokmodus == 4) {
    TijdDatumInstellen(2); //0=niets, 1=SetKnop Lang, 2 = Setknop Kort, 3=Knop boven, 4=knop onder

  }
  else
  {
    Klokmodus++;
    if (Klokmodus >= 3) {
      Klokmodus = 0;
    }
    Serial.print( "Modus: ");
    Serial.println(Klokmodus);
  }
  ontploft = false;
}

void onButton1LongPress() {
  //VanuitRingenOntploffen();
  TijdDatumInstellen(1); //0=niets, 1=SetKnop Lang, 2 = Setknop Kort, 3=Knop boven, 4=knop onder
  Serial.println("long press");

}


void onButton3Pressed() {
  Serial.println("Button3 has been pressed!");
  if (Klokmodus == 4) {
    TijdDatumInstellen(3); //0=niets, 1=SetKnop Lang, 2 = Setknop Kort, 3=Knop boven, 4=knop onder

  }
  else
  {
    if (Klokmodus == 0 or Klokmodus == 1) { //in uur modus brightness aanpassen
      BrightnessPreset = BrightnessPreset + 20;
      if (BrightnessPreset >= 255) {
        BrightnessPreset = 255;
      }
      Serial.print( "Brightness: ");
      Serial.println(BrightnessPreset);
      strip.setBrightness(BrightnessPreset); // Set BRIGHTNESS
      NuUurHash = 7;
      TijdOpLedAanpassen(false);
    }
  }
}


void onButton4Pressed() {
  Serial.println("Button4 has been pressed!");
  if (Klokmodus == 4) {
    TijdDatumInstellen(4); //0=niets, 1=SetKnop Lang, 2 = Setknop Kort, 3=Knop boven, 4=knop onder
  }
  else
  {

    if (Klokmodus == 0 or Klokmodus == 1) { //in uur modus brightness aanpassen
      BrightnessPreset = BrightnessPreset - 20;
      if (BrightnessPreset <= 20) {
        BrightnessPreset = 20;
      }
      Serial.print( "Brightness: ");
      Serial.println(BrightnessPreset);

      strip.setBrightness(BrightnessPreset); // Set BRIGHTNESS
      NuUurHash = 7;
      TijdOpLedAanpassen(false);
    }
  }
}

void VanuitRingenOntploffen() {
  strip.setPixelColor(49, 0, 0, 0 , 0); // ringen uitzetten
  strip.setPixelColor(50, 0, 0, 0 , 0);
  strip.setPixelColor(60, BrightnessPreset , 0, 0 , 0);
  delay(100);
  strip.setPixelColor(60, 0 , 0, 0 , 0);
  strip.show();

  for (int HIndex = 0; HIndex < 5; HIndex++) { // bovenzijde

    box (48 - (HIndex * 12), 2 + (HIndex * 2), true); //Eerste box AAN
    delay (100);
    box (48 - (HIndex * 12), 2 + (HIndex * 2), false); //Eerste box UIT

  }
  NuUurHash = 7;

}

void box (byte StartLED, byte zijde, bool AanUit) {
  for (int HIndex = StartLED; HIndex < (StartLED + zijde ); HIndex++) {// bovenzijde
    if (AanUit) {
      strip.setPixelColor(HIndex, BrightnessPreset , 0, 0 , 0);
    }
    else {
      strip.setPixelColor(HIndex, 0, 0, 0 , 0);
    }
  }
  for (int HIndex = StartLED; HIndex < (StartLED + ((zijde ) * 11) ); HIndex = HIndex + 11) { // linkerzijde
    if (AanUit) {
      strip.setPixelColor(HIndex, BrightnessPreset , 0, 0 , 0);
    }
    else {
      strip.setPixelColor(HIndex, 0, 0, 0 , 0);
    }
  }

  for (int HIndex = StartLED + zijde; HIndex < (StartLED + 1 + zijde + (zijde * 11) ); HIndex = HIndex + 11) { // rechterzijde
    if (AanUit) {
      strip.setPixelColor(HIndex, BrightnessPreset , 0, 0 , 0);
    }
    else {
      strip.setPixelColor(HIndex, 0, 0, 0 , 0);
    }
  }
  for (int HIndex = StartLED + (zijde * 11) ; HIndex < (StartLED + zijde + (zijde * 11) ); HIndex++) { // onderzijde
    if (AanUit) {
      strip.setPixelColor(HIndex, BrightnessPreset , 0, 0 , 0);
    }
    else {
      strip.setPixelColor(HIndex, 0, 0, 0 , 0);
    }
  }
  strip.show();
}

void setPixelWitAdjusted(byte HIndex, byte HKleur ) {  //Kleuren aanpassen aan te rode polyester // Klok 2 heeft geen kleurverschillen  //HKleur: 0 gewoon wit, 1,2 ,3, 4... andere kleuren
  byte rood = 0;
  byte groen = 0;
  byte blauw = 0;
  byte wit = BrightnessPreset;

  if (HKleur == 1) {
    rood = BrightnessPreset;
    groen = 0;
    blauw = BrightnessPreset;
    wit = 0;
  }
  else if (HKleur == 2) {
    rood = BrightnessPreset;
    groen = BrightnessPreset;
    blauw = 0;
    wit = 0;
  }
  else if (HKleur == 3) {
    rood = 0;
    groen = BrightnessPreset;
    blauw = BrightnessPreset;
    wit = 0;
  }
  else if (HKleur == 4) {
    rood = BrightnessPreset;
    groen = 0;
    blauw = 0;
    wit = 0;
  }
  else if (HKleur == 5) {
    rood = 0;
    groen = BrightnessPreset;
    blauw = 0;
    wit = 0;
  }



  /*
    if ((HIndex == 49 or HIndex == 50) ) { //De ringen in het midden rood ipv van wit maken
    rood = BrightnessPreset;
    wit = 0;
    }
    if (HIndex == 109 or HIndex == 66 or HIndex == 67 or HIndex == 68 or HIndex == 69 or HIndex == 72 or HIndex == 77 or HIndex == 78 or HIndex == 81 or HIndex == 82 or HIndex == 83 or HIndex == 84 or HIndex == 85 or HIndex == 87
      or HIndex == 88 or HIndex == 89 or HIndex == 90 or HIndex == 91 or HIndex == 92 or HIndex == 93 or HIndex == 94 or HIndex == 95 or HIndex == 96 or HIndex == 100 or HIndex == 102 or HIndex == 107) { //De kleuren van te rode polyester aanpaasen door extra groen & blauw
    wit = BrightnessPreset;
    blauw = floor (BrightnessPreset);
    groen = floor (BrightnessPreset);
    }
    if (HIndex == 3 or HIndex == 13 or HIndex == 36 or HIndex == 86) { //De kleuren van te rode polyester aanpaasen door extra groen & blauw
    wit = BrightnessPreset;
    blauw = floor (BrightnessPreset / 2);
    groen = floor (BrightnessPreset / 2);
    }
  */
  strip.setPixelColor(HIndex, rood, groen, blauw, wit);
}


void JKDW() {
  if (NuUurHash != 1) { //nakijken of de uur hash overeenkomt met de vorige anders klok updaten
    strip.clear();         //   Set all pixels in RAM to 0 (off)
    byte Janletters[] = {77, 88, 99};  //Jan
    RandomLettersNaarLed(Janletters, 3, true, 5);
    delay(300);
    byte KBletters[] = {11, 22, 33, 44, 45, 56, 67, 78};  //Kathleen
    RandomLettersNaarLed(KBletters, 8, true, 1);
    delay(300);
    TekstNaarLed(58, 4, true, 4); //Daan
    delay(300);
    TekstNaarLed(79, 4, true, 2); //ward
    NuUurHash = 1;
  }
}


void EmmaRoger (bool Anim) {
  if (NuUurHash != 1) { //nakijken of de uur hash overeenkomt met de vorige anders klok updaten
    strip.clear();         //   Set all pixels in RAM to 0 (off)
    TekstNaarLed(37, 4, Anim, 0); //Emma
    TekstNaarLed(49, 2, Anim, 0); //ringen
    TekstNaarLed(58, 5, Anim, 0); //Roger
    if (VerjaardagVandaag == 10 ) { //exacte huwelijksverjaardag
      NummerNaarLed(NuJaar - HuwelijkJaar, true, 0);
      TekstNaarLed(102, 4, Anim, 0); //jaar
    }
    else if (VerjaardagVandaag == 11 ) { //exacte huwelijksverjaardag
      NummerNaarLed(NuUur - HuwelijkUur, true, 0);
      TekstNaarLed(99, 3, Anim, 0); //uur
    }
    else if (VerjaardagVandaag == 12 ) { //eerste huwelijksdagen
      NummerNaarLed(NuDag - HuwelijkDag, true, 0);
      TekstNaarLed(96, 3, Anim, 0); //dag
    }
    else if (VerjaardagVandaag == 13 ) { //eerste huwelijksmaanden
      NummerNaarLed(NuMaand - HuwelijkMaand, true, 0);
      byte Maandletters[] = {88, 79, 80, 95, 96};
      RandomLettersNaarLed(Maandletters, 5, true, 0);
    }
    else {
      TekstNaarLed(66, 4, Anim, 0); //2
      TekstNaarLed(88, 3, Anim, 0); //mei
    }
    NuUurHash = 1;
  }
}

void TijdOpLedAanpassen (bool Animatie) {
  RtcDateTime now = Rtc.GetDateTime();//Tijd opvragen
  NuSeconden = now.Second();
  // Goede tekst samenstellen om op het scherm te tonen
  if ((VerjaardagVandaag > 0) && (NuSeconden % (VerjaardagVervers * 2) > VerjaardagVervers) ) {
    if (VerjaardagVandaag < 10) {
      switch ((int)round((NuSeconden % 4 - 0.2 ) / 2)) { //moeilijke formule om om de 2 seconden te verwisselen
        case 0:
          VanuitRingenOntploffen();
          break;
        case 1:
          if (VerjaardagVandaag == 1) {
            byte Minuutletters[] = {77, 88, 99};  //Jan
            RandomLettersNaarLed(Minuutletters, 3, true, 0);
          }
          else if (VerjaardagVandaag == 2) {
            byte Minuutletters[] = {11, 22, 33, 44, 45, 56, 67, 78};  //Kathleen
            RandomLettersNaarLed(Minuutletters, 8, true, 0);
          }
          else if (VerjaardagVandaag == 3) {
            TekstNaarLed(58, 4, true, 0); //Daan
          }
          else if (VerjaardagVandaag == 4) {
            TekstNaarLed(79, 4, true, 0); //ward
          }

          break;
      }
      NuUurHash = "x"; //NuUurHash aanpassen zodat tijd nadien weergegeven wordt
    }
    else if (VerjaardagVandaag >= 10) {
      EmmaRoger (true);
    }
  }
  else {
    if (NuUurHash != (floor(now.Minute() / 5) + (now.Hour() * 100) )) { //nakijken of de uur hash overeenkomt met de vorige anders klok updaten
      strip.clear();         //   Set all pixels in RAM to 0 (off)
      UurNaarLED(now.Hour(), now.Minute(), true , Animatie); //uur, minuten, HetIs?, Animatie?
      NuUurHash = (floor(now.Minute() / 5) + (now.Hour() * 100) ); //UurHash updaten
    }
  }
}

void NummerNaarLed(byte Nummer, bool Animatie, byte HKleur) {  //individuele nummer op de klok zetten
  if (Nummer < 13) {
    for (int HIndex = NummersLocatie[Nummer]; HIndex < (NummersLengte[Nummer] + NummersLocatie[Nummer] ); HIndex++) {
      setPixelWitAdjusted (HIndex, HKleur);
      if (Animatie) {
        delay(100); // Pause 1/2 second
        strip.show(); //Laat de kleuren zien!
      }
      else {
        strip.show();
      }
    }
  }
  else if (Nummer < 21) {
    TekstNaarLed (22, 4, Animatie, 0);  //10
    byte Nummer10 = Nummer - 10;
    for (int HIndex = NummersLocatie[Nummer10]; HIndex < (NummersLengte[Nummer10] + NummersLocatie[Nummer10] ); HIndex++) {
      setPixelWitAdjusted (HIndex, HKleur);
      if (Animatie) {
        delay(100); // Pause 1/2 second
        strip.show(); //Laat de kleuren zien!
      }
      else {
        strip.show();
      }
    }


  }
  else if (Nummer < 32) {
    TekstNaarLed (15, 7, Animatie, 0);  //20
    byte Nummer20 = Nummer - 20;
    for (int HIndex = NummersLocatie[Nummer20]; HIndex < (NummersLengte[Nummer20] + NummersLocatie[Nummer20] ); HIndex++) {
      setPixelWitAdjusted (HIndex, HKleur);
      if (Animatie) {
        delay(100); // Pause 1/2 second
        strip.show(); //Laat de kleuren zien!
      }
      else {
        strip.show();
      }
    }


  }
}

void TekstNaarLed(byte StartLED, byte LengteWoord, bool Animatie, byte HKleur) { // groep aaneensluitende letters op de klok zetten
  for (int HIndex = StartLED; HIndex < (StartLED + LengteWoord ); HIndex++) {
    setPixelWitAdjusted (HIndex, HKleur);
    if (Animatie) {
      delay(100); // Pause 1/2 second
      strip.show(); //Laat de kleuren zien!
    }
    strip.show();
  }
}

void RandomLettersNaarLed(byte HIndex[], byte lengte, bool Animatie, byte HKleur) { // groep niet-aaneensluitende letters op de klok zetten
  for (byte i = 0 ; i < lengte; i++) {
    Serial.print(sizeof (HIndex));
    setPixelWitAdjusted (HIndex[i], HKleur);
    if (Animatie) {
      delay(100); // Pause 1/2 second
      strip.show(); //Laat de kleuren zien!
    }
    strip.show();
  }
}


void UurNaarLED(int Uur, int Min, bool HetIs, bool Animatie) { //uur op de klok zetten
  bool PM = false;
  byte Voor[] = {27, 4};
  byte Na[] = {31, 2};
  byte vijf[] = {7, 4};
  byte tien[] = {23, 4};
  byte Kwart[] = {11, 5};
  byte Half[] = {34, 4};
  byte Twintig[] = {15, 7};

  byte uur[] =  {103, 3};

  if (Uur >= 12) {
    PM = true;
  }
  if (Min >= 25) {
    Uur = Uur + 1;
  }
  if (Uur > 12)  {
    Uur = Uur - 12;
  }
  if (Uur == 0) {
    Uur = 12;
  }
  //String UurTekst = nummers[Uur];
  if (HetIs) {
    TekstNaarLed (0, 3, Animatie, 0); //print "het"
    TekstNaarLed (4, 2, Animatie, 0); //print "is"

  }

  Min = floor(Min / 5);
  switch (Min) {
    case 0: NummerNaarLed(Uur, Animatie, 0 ); TekstNaarLed (uur[0], uur[1], Animatie, 0) ;    break;
    case 1: TekstNaarLed (vijf[0], vijf[1], Animatie, 0);   TekstNaarLed (Na[0], Na[1], Animatie, 0); NummerNaarLed(Uur, Animatie, 0 );    break;
    case 2: TekstNaarLed (tien[0], tien[1], Animatie, 0);   TekstNaarLed (Na[0], Na[1], Animatie, 0); NummerNaarLed(Uur, Animatie, 0 );    break;
    case 3: TekstNaarLed (Kwart[0], Kwart[1], Animatie, 0); TekstNaarLed (Na[0], Na[1], Animatie, 0); NummerNaarLed(Uur, Animatie, 0 );   break;
    case 4: TekstNaarLed (Twintig[0], Twintig[1], Animatie, 0); TekstNaarLed (Na[0], Na[1], Animatie, 0); NummerNaarLed(Uur, Animatie, 0 );     break;
    case 5: TekstNaarLed (vijf[0], vijf[1], Animatie, 0); TekstNaarLed (Voor[0], Voor[1], Animatie, 0); TekstNaarLed (Half[0], Half[1], Animatie, 0); NummerNaarLed(Uur, Animatie, 0 );   break;
    case 6: TekstNaarLed (Half[0], Half[1], Animatie, 0); NummerNaarLed(Uur, Animatie, 0 );    break;
    case 7: TekstNaarLed (vijf[0], vijf[1], Animatie, 0); TekstNaarLed (Na[0], Na[1], Animatie, 0); TekstNaarLed (Half[0], Half[1], Animatie, 0); NummerNaarLed(Uur, Animatie, 0 );   break;
    case 8: TekstNaarLed (Twintig[0], Twintig[1], Animatie, 0); TekstNaarLed (Voor[0], Voor[1], Animatie, 0); NummerNaarLed(Uur, Animatie, 0 );    break;
    case 9: TekstNaarLed (Kwart[0], Kwart[1], Animatie, 0); TekstNaarLed (Voor[0], Voor[1], Animatie, 0); NummerNaarLed(Uur, Animatie, 0 ); break;
    case 10: TekstNaarLed (tien[0], tien[1], Animatie, 0); TekstNaarLed (Voor[0], Voor[1], Animatie, 0); NummerNaarLed(Uur, Animatie, 0 ); break;
    case 11: TekstNaarLed (vijf[0], vijf[1], Animatie, 0); TekstNaarLed (Voor[0], Voor[1], Animatie, 0); NummerNaarLed(Uur, Animatie, 0 ); break;
    default: TekstNaarLed (49, 2, Animatie, 0);    break;
  }
  if (PM) {

    TekstNaarLed (108, 2, Animatie, 0); //print " PM"
  }
  else {
    if (Min == 0) { // Op de volle uren in de voormiddag geen AM op het scherm zetten, want de spatie ontbreekt
      ;
    }
    else {
    TekstNaarLed (106, 2, Animatie, 0); //print " AM"
    }
  }

}

void printArray(int* x, int length, char separator)
{
  for (int iCount = 0; iCount < length - 1; iCount++)
  {
    Serial.print(x[iCount]);
    Serial.print(separator);
  }
  Serial.print(x[length - 1]);
  Serial.println();
}

void RTCTest() {

  if (!Rtc.IsDateTimeValid()) { //Alles voor fout debugging van RTC (kopie van voorbeeld Makuna
    if (Rtc.LastError() != 0) {
      // we have a communications error
      // see https://www.arduino.cc/en/Reference/WireEndTransmission for
      // what the number means
      Serial.print("RTC communications error = ");
      Serial.println(Rtc.LastError());
    }
    else {
      // Common Causes:
      //    1) the battery on the device is low or even missing and the power line was disconnected
      Serial.println("RTC lost confidence in the DateTime!");
    }
  }
}
